@@ -1,43 +1,58 @@
 """
 Unittests for the functions module
+
+TODO: format_number
+TODO: now
+TODO: check_if_in_shell
+TODO: timedelta
+TODO: get_ANSI_colours
+TODO: mem_use
+TODO: trem
+TODO: conv_time_units
+TODO: convert_bytes
+TODO: get_size
+TODO: imports
+TODO: isfloat
+TODO: isint
+TODO: convfloat
+TODO: datalinedict
+TODO: pad_output_distribution
+TODO: catchtime
+TODO: is_capsule
+TODO: Capturing
+TODO: call_binary_c_config
 """
 
+import os
 import unittest
 import tempfile
-from binarycpython.utils.functions import *
+
+import json
+import h5py
+
 from binarycpython.utils.custom_logging_functions import binary_c_log_code
 from binarycpython.utils.run_system_wrapper import run_system
+from binarycpython.utils.functions import (
+    verbose_print,
+    temp_dir,
+    Capturing,
+    remove_file,
+    create_hdf5,
+    output_lines,
+    example_parse_output,
+    get_defaults,
+    get_arg_keys,
+    create_arg_string,
+    get_help,
+    get_help_all,
+    get_help_super,
+    make_build_text,
+    write_binary_c_parameter_descriptions_to_rst_file,
+    get_username,
+    bin_data,
+)
 
-binary_c_temp_dir = temp_dir()
-
-#############################
-# Script that contains unit tests for functions from the binarycpython.utils.functions file
-
-# class test_(unittest.TestCase):
-#     """
-#     Unittests for function
-#     """
-
-#     def test_1(self):
-#         pass
-
-
-class dummy:
-    """
-    Dummy class to be used in the merge_dicts
-    """
-
-    def __init__(self, name):
-        """
-        init
-        """
-        self.name = name
-
-    def __str__(self):
-        """
-        str returns self.name
-        """
-        return self.name
+TMP_DIR = temp_dir("tests", "test_functions")
 
 
 class test_verbose_print(unittest.TestCase):
@@ -46,12 +61,20 @@ class test_verbose_print(unittest.TestCase):
     """
 
     def test_print(self):
+        with Capturing() as output:
+            self._test_print()
+
+    def _test_print(self):
         """
         Tests whether something gets printed
         """
         verbose_print("test1", 1, 0)
 
     def test_not_print(self):
+        with Capturing() as output:
+            self._test_not_print()
+
+    def _test_not_print(self):
         """
         Tests whether nothing gets printed.
         """
@@ -59,65 +82,88 @@ class test_verbose_print(unittest.TestCase):
         verbose_print("test1", 0, 1)
 
 
-class test_remove_file(unittest.TestCase):
-    """
-    Unittests for remove_file
-    """
-
-    def test_remove_file(self):
-        """
-        Test to remove a file
-        """
-
-        with open(
-            os.path.join(binary_c_temp_dir, "test_remove_file_file.txt"), "w"
-        ) as f:
-            f.write("test")
-
-        remove_file(os.path.join(binary_c_temp_dir, "test_remove_file_file.txt"))
-
-    def test_remove_nonexisting_file(self):
-        """
-        Test to try to remove a nonexistant file
-        """
-
-        file = os.path.join(binary_c_temp_dir, "test_remove_nonexistingfile_file.txt")
-
-        remove_file(file)
-
-
 class test_temp_dir(unittest.TestCase):
     """
     Unittests for temp_dir
     """
 
     def test_create_temp_dir(self):
+        with Capturing() as output:
+            self._test_create_temp_dir()
+
+    def _test_create_temp_dir(self):
         """
         Test making a temp directory and comparing that to what it should be
         """
 
+        #
         binary_c_temp_dir = temp_dir()
+        username = get_username()
         general_temp_dir = tempfile.gettempdir()
 
+        # Get username
+        username = get_username()
+        dirname = "binary_c_python-{}".format(username)
+
+        #
         self.assertTrue(
-            os.path.isdir(os.path.join(general_temp_dir, "binary_c_python"))
+            os.path.isdir(
+                os.path.join(general_temp_dir, "binary_c_python-{}".format(username))
+            )
         )
         self.assertTrue(
-            os.path.join(general_temp_dir, "binary_c_python")
+            os.path.join(general_temp_dir, "binary_c_python-{}".format(username))
         ) == binary_c_temp_dir
 
 
+class test_remove_file(unittest.TestCase):
+    """
+    Unittests for remove_file
+    """
+
+    def test_remove_file(self):
+        with Capturing() as output:
+            self._test_remove_file()
+
+    def _test_remove_file(self):
+        """
+        Test to remove a file
+        """
+
+        with open(os.path.join(TMP_DIR, "test_remove_file_file.txt"), "w") as f:
+            f.write("test")
+
+        remove_file(os.path.join(TMP_DIR, "test_remove_file_file.txt"))
+
+    def test_remove_nonexisting_file(self):
+        with Capturing() as output:
+            self._test_remove_nonexisting_file()
+
+    def _test_remove_nonexisting_file(self):
+        """
+        Test to try to remove a nonexistant file
+        """
+
+        file = os.path.join(TMP_DIR, "test_remove_nonexistingfile_file.txt")
+
+        remove_file(file)
+
+
 class test_create_hdf5(unittest.TestCase):
     """
     Unittests for create_hdf5
     """
 
     def test_1(self):
+        with Capturing() as output:
+            self._test_1()
+
+    def _test_1(self):
         """
         Test that creates files, packs them in a hdf5 file and checks the contents
         """
 
-        testdir = os.path.join(binary_c_temp_dir, "test_create_hdf5")
+        testdir = os.path.join(TMP_DIR, "test_create_hdf5")
         os.makedirs(testdir, exist_ok=True)
 
         # Create dummy settings file:
@@ -140,81 +186,16 @@ class test_create_hdf5(unittest.TestCase):
         self.assertIn("settings_2", json.loads(file.get("settings/used_settings")[()]))
 
 
-class test_return_binary_c_version_info(unittest.TestCase):
-    """
-    Unittests for return_binary_c_version_info
-    """
-
-    def test_not_parsed(self):
-        """
-        Test for the raw version_info output
-        """
-
-        version_info = return_binary_c_version_info()
-
-        self.assertTrue(isinstance(version_info, str))
-        self.assertIn("Build", version_info)
-        self.assertIn("REIMERS_ETA_DEFAULT", version_info)
-        self.assertIn("SIGMA_THOMPSON", version_info)
-
-    def test_parsed(self):
-        """
-        Test for the parssed version_info
-        """
-
-        # also tests the parse_version_info indirectly
-        version_info_parsed = return_binary_c_version_info(parsed=True)
-
-        self.assertTrue(isinstance(version_info_parsed, dict))
-        self.assertIn("isotopes", version_info_parsed.keys())
-        self.assertIn("argpairs", version_info_parsed.keys())
-        self.assertIn("ensembles", version_info_parsed.keys())
-        self.assertIn("macros", version_info_parsed.keys())
-        self.assertIn("elements", version_info_parsed.keys())
-        self.assertIn("dt_limits", version_info_parsed.keys())
-        self.assertIn("nucleosynthesis_sources", version_info_parsed.keys())
-        self.assertIn("miscellaneous", version_info_parsed.keys())
-
-
-class test_parse_binary_c_version_info(unittest.TestCase):
-    """
-    Unittests for function parse_binary_c_version_info
-    """
-
-    def test_1(self):
-        """
-        Test for the parsed versio info, more detailed
-        """
-
-        info = return_binary_c_version_info()
-        parsed_info = parse_binary_c_version_info(info)
-
-        self.assertIn("isotopes", parsed_info.keys())
-        self.assertIn("argpairs", parsed_info.keys())
-        self.assertIn("ensembles", parsed_info.keys())
-        self.assertIn("macros", parsed_info.keys())
-        self.assertIn("elements", parsed_info.keys())
-        self.assertIn("dt_limits", parsed_info.keys())
-        self.assertIn("nucleosynthesis_sources", parsed_info.keys())
-        self.assertIn("miscellaneous", parsed_info.keys())
-
-        self.assertIsNotNone(parsed_info["argpairs"])
-        self.assertIsNotNone(parsed_info["ensembles"])
-        self.assertIsNotNone(parsed_info["macros"])
-        self.assertIsNotNone(parsed_info["dt_limits"])
-        self.assertIsNotNone(parsed_info["miscellaneous"])
-
-        if parsed_info["macros"]["NUCSYN"] == "on":
-            self.assertIsNotNone(parsed_info["isotopes"])
-            self.assertIsNotNone(parsed_info["nucleosynthesis_sources"])
-
-
 class test_output_lines(unittest.TestCase):
     """
     Unittests for function output_lines
     """
 
     def test_1(self):
+        with Capturing() as output:
+            self._test_1()
+
+    def _test_1(self):
         """
         Test to check if the shape and contents of output_lines is correct
         """
@@ -234,6 +215,10 @@ class test_example_parse_output(unittest.TestCase):
     """
 
     def test_normal_output(self):
+        with Capturing() as output:
+            self._test_normal_output()
+
+    def _test_normal_output(self):
         """
         Test checking if parsed output with a custom logging line works correctly
         """
@@ -263,6 +248,10 @@ class test_example_parse_output(unittest.TestCase):
         self.assertTrue(len(parsed_output["time"]) > 0)
 
     def test_mismatch_output(self):
+        with Capturing() as output:
+            self._test_mismatch_output()
+
+    def _test_mismatch_output(self):
         """
         Test checking if parsed output with a mismatching headerline doesnt have any contents
         """
@@ -294,6 +283,10 @@ class test_get_defaults(unittest.TestCase):
     """
 
     def test_no_filter(self):
+        with Capturing() as output:
+            self._test_no_filter()
+
+    def _test_no_filter(self):
         """
         Test checking if the defaults without filtering contains non-filtered content
         """
@@ -306,6 +299,10 @@ class test_get_defaults(unittest.TestCase):
         self.assertIn("list_args", output_1.keys())
         self.assertIn("use_fixed_timestep_%d", output_1.keys())
 
+    def test_filter(self):
+        with Capturing() as output:
+            self._test_filter()
+
     def test_filter(self):
         """
         Test checking filtering works correctly
@@ -327,6 +324,10 @@ class test_get_arg_keys(unittest.TestCase):
     """
 
     def test_1(self):
+        with Capturing() as output:
+            self._test_1()
+
+    def _test_1(self):
         """
         Test checking if some of the keys are indeed in the list
         """
@@ -346,6 +347,10 @@ class test_create_arg_string(unittest.TestCase):
     """
 
     def test_default(self):
+        with Capturing() as output:
+            self._test_default()
+
+    def _test_default(self):
         """
         Test checking if the argstring is correct
         """
@@ -355,6 +360,10 @@ class test_create_arg_string(unittest.TestCase):
         self.assertEqual(argstring, "separation 40000 M_1 10")
 
     def test_sort(self):
+        with Capturing() as output:
+            self._test_sort()
+
+    def _test_sort(self):
         """
         Test checking if the argstring with a different ordered dict is also in a differnt order
         """
@@ -364,6 +373,10 @@ class test_create_arg_string(unittest.TestCase):
         self.assertEqual(argstring, "M_1 10 separation 40000")
 
     def test_filtered(self):
+        with Capturing() as output:
+            self._test_filtered()
+
+    def _test_filtered(self):
         """
         Test if filtering works
         """
@@ -379,6 +392,10 @@ class test_get_help(unittest.TestCase):
     """
 
     def test_input_normal(self):
+        with Capturing() as output:
+            self._test_input_normal()
+
+    def _test_input_normal(self):
         """
         Function to test the get_help function
         """
@@ -390,6 +407,10 @@ class test_get_help(unittest.TestCase):
         )
 
     def test_no_input(self):
+        with Capturing() as output:
+            self._test_no_input()
+
+    def _test_no_input(self):
         """
         Test if the result is None if called without input
         """
@@ -398,15 +419,16 @@ class test_get_help(unittest.TestCase):
         self.assertIsNone(output)
 
     def test_wrong_input(self):
+        with Capturing() as output:
+            self._test_wrong_input()
+
+    def _test_wrong_input(self):
         """
         Test if the result is None if called with an unknown input
         """
 
         output = get_help("kaasblokjes")
-        self.assertIsNone(output)
-
-    # def test_print(self):
-    #     output = get_help("M_1", print_help=True)
+        self.assertDictEqual(output, {})
 
 
 class test_get_help_all(unittest.TestCase):
@@ -415,6 +437,10 @@ class test_get_help_all(unittest.TestCase):
     """
 
     def test_all_output(self):
+        with Capturing() as output:
+            self._test_all_output()
+
+    def _test_all_output(self):
         """
         Function to test the get_help_all function
         """
@@ -441,6 +467,10 @@ class test_get_help_super(unittest.TestCase):
     """
 
     def test_all_output(self):
+        with Capturing() as output:
+            self._test_all_output()
+
+    def _test_all_output(self):
         """
         Function to test the get_help_super function
         """
@@ -467,6 +497,10 @@ class test_make_build_text(unittest.TestCase):
     """
 
     def test_output(self):
+        with Capturing() as output:
+            self._test_output()
+
+    def _test_output(self):
         """
         Test checking the contents of the build_text
         """
@@ -493,218 +527,80 @@ class test_write_binary_c_parameter_descriptions_to_rst_file(unittest.TestCase):
     """
 
     def test_bad_outputname(self):
+        with Capturing() as output:
+            self._test_bad_outputname()
+
+    def _test_bad_outputname(self):
         """
         Test checking if None is returned when a bad input name is provided
         """
 
         output_name = os.path.join(
-            binary_c_temp_dir,
+            TMP_DIR,
             "test_write_binary_c_parameter_descriptions_to_rst_file_test_1.txt",
         )
-        output_1 = write_binary_c_parameter_descriptions_to_rst_file(output_name)
-        self.assertIsNone(output_1)
+        self.assertRaises(
+            ValueError, write_binary_c_parameter_descriptions_to_rst_file, output_name
+        )
 
     def test_checkfile(self):
+        with Capturing() as output:
+            self._test_checkfile()
+
+    def _test_checkfile(self):
         """
         Test checking if the file is created correctly
         """
 
         output_name = os.path.join(
-            binary_c_temp_dir,
+            TMP_DIR,
             "test_write_binary_c_parameter_descriptions_to_rst_file_test_1.rst",
         )
         output_1 = write_binary_c_parameter_descriptions_to_rst_file(output_name)
         self.assertTrue(os.path.isfile(output_name))
 
 
-class test_inspect_dict(unittest.TestCase):
+class test_bin_data(unittest.TestCase):
     """
-    Unittests for function inspect_dict
+    Unittests for bin_data
     """
 
-    def test_compare_dict(self):
+    def test_positive_bin(self):
         """
-        Test checking if inspect_dict returns the correct structure by comparing it to known value
+        Tests to see if the binning is done correctly for positive values
         """
 
-        input_dict = {
-            "int": 1,
-            "float": 1.2,
-            "list": [1, 2, 3],
-            "function": os.path.isfile,
-            "dict": {"int": 1, "float": 1.2},
-        }
-        output_dict = inspect_dict(input_dict)
-        compare_dict = {
-            "int": int,
-            "float": float,
-            "list": list,
-            "function": os.path.isfile.__class__,
-            "dict": {"int": int, "float": float},
-        }
-        self.assertTrue(compare_dict == output_dict)
+        value = 0.6
+        binwidth = 1
 
-    def test_compare_dict_with_print(self):
+        binned_value = bin_data(value, binwidth)
+
+        self.assertEqual(binned_value, 0.5)
+
+    def test_negative_bin(self):
         """
-        Test checking output is printed
+        Tests to see if the binning is done correctly for negative values
         """
 
-        input_dict = {
-            "int": 1,
-            "float": 1.2,
-            "list": [1, 2, 3],
-            "function": os.path.isfile,
-            "dict": {"int": 1, "float": 1.2},
-        }
-        output_dict = inspect_dict(input_dict, print_structure=True)
+        value = -0.6
+        binwidth = 1
 
+        binned_value = bin_data(value, binwidth)
 
-class test_merge_dicts(unittest.TestCase):
-    """
-    Unittests for function merge_dicts
-    """
+        self.assertEqual(binned_value, -0.5)
 
-    def test_empty(self):
+    def test_zero_bin(self):
         """
-        Test merging an empty dict
+        Tests to see if the binning is done correctly
+        TODO: when the value is 0 then its now binned in the negative located bin. Decide whether we want that
         """
 
-        input_dict = {
-            "int": 1,
-            "float": 1.2,
-            "list": [1, 2, 3],
-            "function": os.path.isfile,
-            "dict": {"int": 1, "float": 1.2},
-        }
-        dict_2 = {}
-        output_dict = merge_dicts(input_dict, dict_2)
-        self.assertTrue(output_dict == input_dict)
+        value = 0
+        binwidth = 1
 
-    def test_unequal_types(self):
-        """
-        Test merging unequal types: should raise valueError
-        """
+        binned_value = bin_data(value, binwidth)
 
-        dict_1 = {"input": 10}
-        dict_2 = {"input": "hello"}
-
-        self.assertRaises(ValueError, merge_dicts, dict_1, dict_2)
-
-    def test_bools(self):
-        """
-        Test merging dict with booleans
-        """
-
-        dict_1 = {"bool": True}
-        dict_2 = {"bool": False}
-        output_dict = merge_dicts(dict_1, dict_2)
-
-        self.assertTrue(isinstance(output_dict["bool"], bool))
-        self.assertTrue(output_dict["bool"])
-
-    def test_ints(self):
-        """
-        Test merging dict with ints
-        """
-
-        dict_1 = {"int": 2}
-        dict_2 = {"int": 1}
-        output_dict = merge_dicts(dict_1, dict_2)
-
-        self.assertTrue(isinstance(output_dict["int"], int))
-        self.assertEqual(output_dict["int"], 3)
-
-    def test_floats(self):
-        """
-        Test merging dict with floats
-        """
-
-        dict_1 = {"float": 4.5}
-        dict_2 = {"float": 4.6}
-        output_dict = merge_dicts(dict_1, dict_2)
-
-        self.assertTrue(isinstance(output_dict["float"], float))
-        self.assertEqual(output_dict["float"], 9.1)
-
-    def test_lists(self):
-        """
-        Test merging dict with lists
-        """
-
-        dict_1 = {"list": [1, 2]}
-        dict_2 = {"list": [3, 4]}
-        output_dict = merge_dicts(dict_1, dict_2)
-
-        self.assertTrue(isinstance(output_dict["list"], list))
-        self.assertEqual(output_dict["list"], [1, 2, 3, 4])
-
-    def test_dicts(self):
-        """
-        Test merging dict with dicts
-        """
-
-        dict_1 = {"dict": {"same": 1, "other_1": 2.0}}
-        dict_2 = {"dict": {"same": 2, "other_2": [4.0]}}
-        output_dict = merge_dicts(dict_1, dict_2)
-
-        self.assertTrue(isinstance(output_dict["dict"], dict))
-        self.assertEqual(
-            output_dict["dict"], {"same": 3, "other_1": 2.0, "other_2": [4.0]}
-        )
-
-    def test_unsupported(self):
-        """
-        Test merging dict with unsupported types. should raise ValueError
-        """
-
-        dict_1 = {"new": dummy("david")}
-        dict_2 = {"new": dummy("gio")}
-
-        # output_dict = merge_dicts(dict_1, dict_2)
-        self.assertRaises(ValueError, merge_dicts, dict_1, dict_2)
-
-
-class test_binaryc_json_serializer(unittest.TestCase):
-    """
-    Unittests for function binaryc_json_serializer
-    """
-
-    def test_not_function(self):
-        """
-        Test passing an object that doesnt get turned in to a string
-        """
-
-        stringo = "hello"
-        output = binaryc_json_serializer(stringo)
-        self.assertTrue(stringo == output)
-
-    def test_function(self):
-        """
-        Test passing an object that gets turned in to a string: a function
-        """
-
-        string_of_function = str(os.path.isfile)
-        output = binaryc_json_serializer(os.path.isfile)
-        self.assertTrue(string_of_function == output)
-
-
-class test_handle_ensemble_string_to_json(unittest.TestCase):
-    """
-    Unittests for function handle_ensemble_string_to_json
-    """
-
-    def test_1(self):
-        """
-        Test passing string representation of a dictionary.
-        """
-
-        string_of_function = str(os.path.isfile)
-        input_string = '{"ding": 10, "list_example": [1,2,3]}'
-        output_dict = handle_ensemble_string_to_json(input_string)
-
-        self.assertTrue(isinstance(output_dict, dict))
-        self.assertTrue(output_dict["ding"] == 10)
-        self.assertTrue(output_dict["list_example"] == [1, 2, 3])
+        self.assertEqual(binned_value, -0.5)
 
 
 if __name__ == "__main__":
