@@ -8,52 +8,64 @@ Tasks:
     - TODO: change all prints to verbose_prints
 """
 
-
-import bz2
+import os
+import sys
+import time
+import json
+import datetime
 import collections
-from colorama import Fore, Back, Style
-import copy
-import datetime as dt
-import gc
-import gzip
-from halo import Halo
+import resource
+import tempfile
+import subprocess
+import types
+
+from typing import Union
+from io import StringIO
+
 import h5py
 import humanize
-import inspect
-from io import StringIO
-import json
-import msgpack
 import numpy as np
-import os
 import psutil
-import py_rinterpolate
-import re
-import resource
-import sys
-import subprocess
-import tempfile
-import time
-import types
-from typing import Union, Any
-
-import simplejson
-
-# import orjson
-
-import astropy.units as u
-import binarycpython.utils.moe_di_stefano_2017_data as moe_di_stefano_2017_data
+from colorama import Fore, Back, Style
 
 from binarycpython import _binary_c_bindings
-from binarycpython.utils.dicts import filter_dict, filter_dict_through_values
-
+from binarycpython.utils.dicts import filter_dict_through_values
 
 ########################################################
 # Unsorted
 ########################################################
 
 
+def now(now_object=None, style=None, custom_format=None):
+    """
+    convenience function to return a string of the current time, using the format ``%m/%d/%Y %H:%M:%S``
+
+    Args:
+        style : if "nospace" then return the date/time with the format ``%Y%m%d_%H%M%S``, else use format ``%m/%d/%Y %H:%M:%S``
+
+        custom_format: if set, uses this as a format rather than whatever is set by default or in the style variable
+    """
+
+    if not now_object:
+        now_object = datetime.datetime.now()
+
+    if not custom_format:
+        if style == "nospace":
+            # special case
+            date_format = "%Y%m%d_%H%M%S"
+        else:
+            # our default
+            date_format = "%m/%d/%Y %H:%M:%S"
+    else:
+        date_format = custom_format
+
+    return datetime.datetime.strftime(now_object, date_format)
+
+
 def format_number(number):
-    # compact number formatter
+    """
+    Function to take a number, express format it in scientific notation, and remove the trailing 0 if the exponent is 0
+    """
 
     string = "{number:.2g}".format(number=number)
     string = string.replace("e+0", "e+")
@@ -67,12 +79,7 @@ def check_if_in_shell():
     Function to check whether the script is running from a shell
     """
 
-    if sys.stdin and sys.stdin.isatty():
-        in_shell = True
-    else:
-        in_shell = False
-
-    return in_shell
+    return bool(sys.stdin and sys.stdin.isatty())
 
 
 def timedelta(delta):
@@ -82,7 +89,7 @@ def timedelta(delta):
     """
     # currently use the humanize module to do this
     t = humanize.time.precisedelta(
-        dt.timedelta(seconds=delta),
+        datetime.timedelta(seconds=delta),
         format="%0.2f",
         minimum_unit="milliseconds",
         suppress=["milliseconds"],
@@ -100,6 +107,10 @@ def timedelta(delta):
 
 
 def get_ANSI_colours():
+    """
+    Function that returns a dictionary with text-colors in ANSI formatting
+    """
+
     # ANSI colours dictionary
     foreground_colours = {
         "red": Fore.RED,
@@ -110,6 +121,7 @@ def get_ANSI_colours():
         "magenta": Fore.MAGENTA,
         "white": Fore.WHITE,
         "black": Fore.BLACK,
+        "bold": Style.BRIGHT,
     }
 
     background_colours = {
@@ -126,18 +138,19 @@ def get_ANSI_colours():
     default_style = Style.BRIGHT
     colours = {}
 
-    for c in foreground_colours:
-        colours[c] = default_style + foreground_colours[c]
-        for d in background_colours:
-            colours[c + " on " + d] = foreground_colours[c] + background_colours[d]
+    for c, foreground_colour in foreground_colours.items():
+        colours[c] = default_style + foreground_colour
+        for d, background_colour in background_colours.items():
+            colours[c + " on " + d] = foreground_colour + background_colour
     colours["reset"] = Style.RESET_ALL
-
     return colours
 
 
 def mem_use():
     """
-    Return current process memory use in MB. (Takes no arguments) Note: this is per-thread only.
+    Return current process memory use in MB. (Takes no arguments)
+
+    Note: this is per-thread only.
     """
 
     return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss / 1024.0
@@ -145,11 +158,19 @@ def mem_use():
 
 def trem(dt, count, dn, n):
     """
-    Estimate time remaining (seconds) given a differential time and count (i.e. progress = $count/$n). $dt is the time since the last call, $count is the current progress count, $dn is the number run since the last call, and $n is the total number required.
+    Estimate time remaining (seconds) given a differential time and count (i.e. ``progress = count/n``).
+
+    Args:
+        dt: is the time since the last call.
+        count: is the current progress count.
+        dn: is the number run since the last call.
+        n: is the total number required.
     """
+
     tpr = dt / max(1, dn)
     etasecs = tpr * (n - count)
     (eta, units) = conv_time_units(etasecs)
+
     return (eta, units, tpr, etasecs)
 
 
@@ -170,9 +191,16 @@ def conv_time_units(t):
 
 def bin_data(value, binwidth):
     """
-    Function that bins the data
+    Function that bins the data using the absolute value of binwidth using the following formula::
 
-    Uses the absolute value of binwidth
+        ((0.5 if value > 0.0 else -0.5) + int(value / abs(binwidth))) * abs(binwidth)
+
+    Args:
+        value: value that we want to bin
+        binwidth: width of the binning
+
+    Returns:
+        binned value
     """
 
     return ((0.5 if value > 0.0 else -0.5) + int(value / abs(binwidth))) * abs(binwidth)
@@ -204,6 +232,7 @@ def get_size(obj, seen=None):
     obj_id = id(obj)
     if obj_id in seen:
         return 0
+
     # Important mark as seen *before* entering recursion to gracefully handle
     # self-referential objects
     seen.add(obj_id)
@@ -217,69 +246,51 @@ def get_size(obj, seen=None):
     return size
 
 
-def get_moe_di_stefano_dataset(options, verbosity=0):
-    """
-    Function to get the default Moe and di Stefano dataset or accept a user input.
-
-    Returns a dict containing the (JSON) data.
-    """
-
-    json_data = None
-
-    if "JSON" in options:
-        # use the JSON data passed in
-        json_data = options["JSON"]
-
-    elif "file" in options:
-        # use the file passed in, if provided
-        if not os.path.isfile(options["file"]):
-            verbose_print(
-                "The provided 'file' Moe and de Stefano JSON file does not seem to exist at {}".format(
-                    options["file"]
-                ),
-                verbosity,
-                1,
-            )
-
-            raise ValueError
-        if not options["file"].endswith(".json"):
-            verbose_print(
-                "Provided filename is not a json file",
-                verbosity,
-                1,
-            )
-
-        else:
-            # Read input data and Clean up the data if there are white spaces around the keys
-            with open(options["file"], "r") as data_filehandle:
-                datafile_data = data_filehandle.read()
-                datafile_data = datafile_data.replace('" ', '"')
-                datafile_data = datafile_data.replace(' "', '"')
-                datafile_data = datafile_data.replace(' "', '"')
-                json_data = json.loads(datafile_data)
-
-    if not json_data:
-        # no JSON data or filename given, use the default 2017 dataset
-        verbose_print(
-            "Using the default Moe and de Stefano 2017 datafile",
-            verbosity,
-            1,
-        )
-        json_data = copy.deepcopy(moe_di_stefano_2017_data.moe_di_stefano_2017_data)
-
-    return json_data
-
-
 def imports():
-    for name, val in globals().items():
+    """
+    Generator that generates the names of all the modules that are loaded in the globals
+    """
+
+    for _, val in globals().items():
         if isinstance(val, types.ModuleType):
             yield val.__name__
 
 
+def isfloat(x: Union[str, float, int]):
+    """
+    Function to return `True` if the "number" x, which could be a string, is an float, otherwise return `False`.
+
+    Args:
+        x: string float or int that we will attempt to convert to an `float` value.
+    """
+
+    try:
+        _ = float(x)
+        return True
+    except ValueError:
+        return False
+
+
+def isint(x: Union[str, float, int]):
+    """
+    Function to return `True` if the "number" x, which could be a string, is an int, otherwise return `False`.
+
+    Args:
+        x: string float or int that we will attempt to convert to an `int` value.
+    """
+
+    try:
+        _ = int(x)
+        return True
+    except ValueError:
+        return False
+
+
 def convfloat(x):
     """
     Convert scalar x to a float if we can, in which case return the float, otherwise just return x without changing it. Usually, x is a string, but could be anything that float() can handle without failure.
     """
+
     try:
         y = float(x)
         return y
@@ -290,11 +301,13 @@ def convfloat(x):
 def datalinedict(line: str, parameters: list):
     """
     Convert a line of data to a more convenient dictionary.
+
     Arguments:
        line = a line of data as a string
        parameters = a list of the parameter names
 
-    Note: if the parameter is a floating point number, it will be converted to Python's float type.
+    Note:
+        If the parameter is a floating point number, it will be converted to Python's float type.
     """
 
     return {param: convfloat(value) for param, value in zip(parameters, line.split())}
@@ -303,6 +316,10 @@ def datalinedict(line: str, parameters: list):
 def pad_output_distribution(dist: dict, binwidth: float):
     """
     Given a distribution, dist (a dictionary), which should be binned every binwidth (float), fill the distribution with zeros when there is no data. Note: this changes the data in place.
+
+    Args:
+        dist: dictionary containing the distribution data.
+        binwidth: binwidth that is used to fill the distribution with 0 in places where there is no value/key.
     """
 
     # sorted list of the keys
@@ -321,19 +338,19 @@ def pad_output_distribution(dist: dict, binwidth: float):
     return dist
 
 
-class catchtime(object):
+class catchtime:
     """
     Context manager to calculate time spent
     """
 
     def __enter__(self):
         """On entry we start the clock"""
-        self.t = time.clock()
+        self.t = time.process_time()
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, exc_type, exc_val, exc_tb):
         """On exit we stop the clock and measure the time spent"""
-        self.t = time.clock() - self.t
+        self.t = time.process_time() - self.t
         print("Took {}s".format(self.t))
 
 
@@ -370,11 +387,11 @@ def call_binary_c_config(argument):
     """
     Function to interface with the binary_c config file
 
-    input:
-        - argument: argument for the binary_c config
+    Args:
+        argument: argument for the binary_c config
 
     Returns:
-        - raw output of binary_c-config
+        raw output of binary_c-config
     """
 
     BINARY_C_DIR = os.getenv("BINARY_C", None)
@@ -399,21 +416,25 @@ def call_binary_c_config(argument):
 ########################################################
 
 
-def verbose_print(message: str, verbosity: int, minimal_verbosity: int) -> None:
+def verbose_print(
+    message: str, verbosity: int, minimal_verbosity: int, newline: str = "\n"
+) -> None:
     """
     Function that decides whether to print a message based on the current verbosity
-    and its minimum verbosity
-
-    if verbosity is equal or higher than the minimum, then we print
+    and its minimum verbosity. If verbosity is equal or higher than the minimum, then we print.
 
     Args:
-        message: message to print
-        verbosity: current verbosity level
-        minimal_verbosity: threshold verbosity above which to print
+        message: message to print.
+        verbosity: current verbosity level.
+        minimal_verbosity: threshold verbosity above which to print.
+        newline: newline character (or set of characters), defaults to ``\\n`` but ``\\x0d`` (carriage return) might be useful.
     """
 
     if verbosity >= minimal_verbosity:
-        print(message)
+        if newline == "\n":
+            print(message)
+        else:
+            print(message, newline, sep="", end="")
         sys.stdout.flush()
 
 
@@ -451,17 +472,21 @@ def remove_file(file: str, verbosity: int = 0) -> None:
 
 
 def get_username():
+    """
+    Function to get the username of the user that spawned the current process
+    """
+
     return psutil.Process().username()
 
 
-def temp_dir(*args: str) -> str:
+def temp_dir(*child_dirs: str) -> str:
     """
-    Function to create directory within the TMP directory of the file system
+    Function to create directory within the TMP directory of the file system, starting with `/<TMP>/binary_c_python-<username>`
 
     Makes use of os.makedirs exist_ok which requires python 3.2+
 
     Args:
-        function arguments: str input where each next input will be a child of the previous full_path. e.g. temp_dir('tests', 'grid') will become '/tmp/binary_c_python/tests/grid'
+        *child_dirs: str input where each next input will be a child of the previous full_path. e.g. ``temp_dir('tests', 'grid')`` will become ``'/tmp/binary_c_python-<username>/tests/grid'``
 
     Returns:
         the path of a sub directory called binary_c_python in the TMP of the file system
@@ -472,8 +497,8 @@ def temp_dir(*args: str) -> str:
     path = os.path.join(tmp_dir, "binary_c_python-{}".format(username))
 
     # loop over the other paths if there are any:
-    if args:
-        for extra_dir in args:
+    if child_dirs:
+        for extra_dir in child_dirs:
             path = os.path.join(path, extra_dir)
 
     #
@@ -506,21 +531,23 @@ def create_hdf5(data_dir: str, name: str) -> None:
     content_data_dir = os.listdir(data_dir)
 
     # Settings
-    if any([file.endswith("_settings.json") for file in content_data_dir]):
+    if any(file.endswith("_settings.json") for file in content_data_dir):
         print("Adding settings to HDF5 file")
         settings_file = os.path.join(
             data_dir,
             [file for file in content_data_dir if file.endswith("_settings.json")][0],
         )
 
-        with open(settings_file, "r") as settings_file:
+        with open(settings_file, "r", encoding="utf-8") as settings_file:
             settings_json = json.load(settings_file)
 
         # Create settings group
         settings_grp = hdf5_file.create_group("settings")
 
         # Write version_string to settings_group
-        settings_grp.create_dataset("used_settings", data=json.dumps(settings_json))
+        settings_grp.create_dataset(
+            "used_settings", data=json.dumps(settings_json, ensure_ascii=False)
+        )
 
     # Get data files
     data_files = [el for el in content_data_dir if el.endswith(".dat")]
@@ -551,325 +578,6 @@ def create_hdf5(data_dir: str, name: str) -> None:
         hdf5_file.close()
 
 
-########################################################
-# version_info functions
-########################################################
-
-
-def return_binary_c_version_info(parsed: bool = True) -> Union[str, dict]:
-    """
-    Function that returns the version information of binary_c. This function calls the function
-    _binary_c_bindings.return_version_info()
-
-    Args:
-        parsed: Boolean flag whether to parse the version_info output of binary_c. default = False
-
-    Returns:
-        Either the raw string of binary_c or a parsed version of this in the form of a nested
-        dictionary
-    """
-
-    found_prev = False
-    if "BINARY_C_MACRO_HEADER" in os.environ:
-        # the env var is already present. lets save that and put that back later
-        found_prev = True
-        prev_value = os.environ["BINARY_C_MACRO_HEADER"]
-
-    #
-    os.environ["BINARY_C_MACRO_HEADER"] = "macroxyz"
-
-    # Get version_info
-    version_info = _binary_c_bindings.return_version_info().strip()
-
-    # parse if wanted
-    if parsed:
-        version_info = parse_binary_c_version_info(version_info)
-
-    # delete value
-    del os.environ["BINARY_C_MACRO_HEADER"]
-
-    # put stuff back if we found a previous one
-    if found_prev:
-        os.environ["BINARY_C_MACRO_HEADER"] = prev_value
-
-    return version_info
-
-
-def parse_binary_c_version_info(version_info_string: str) -> dict:
-    """
-    Function that parses the binary_c version info. Long function with a lot of branches
-
-    TODO: fix this function. stuff is missing: isotopes, macros, nucleosynthesis_sources
-
-    Args:
-        version_info_string: raw output of version_info call to binary_c
-
-    Returns:
-        Parsed version of the version info, which is a dictionary containing the keys: 'isotopes' for isotope info, 'argpairs' for argument pair info (TODO: explain), 'ensembles' for ensemble settings/info, 'macros' for macros, 'elements' for atomic element info, 'DTlimit' for (TODO: explain), 'nucleosynthesis_sources' for nucleosynthesis sources, and 'miscellaneous' for all those that were not caught by the previous groups. 'git_branch', 'git_build', 'revision' and 'email' are also keys, but its clear what those contain.
-    """
-
-    version_info_dict = {}
-
-    # Clean data and put in correct shape
-    splitted = version_info_string.strip().splitlines()
-    cleaned = {el.strip() for el in splitted if not el == ""}
-
-    ##########################
-    # Network:
-    # Split off all the networks and parse the info.
-
-    networks = {el for el in cleaned if el.startswith("Network ")}
-    cleaned = cleaned - networks
-
-    networks_dict = {}
-    for el in networks:
-        network_dict = {}
-        split_info = el.split("Network ")[-1].strip().split("==")
-
-        network_number = int(split_info[0])
-        network_dict["network_number"] = network_number
-
-        network_info_split = split_info[1].split(" is ")
-
-        shortname = network_info_split[0].strip()
-        network_dict["shortname"] = shortname
-
-        if not network_info_split[1].strip().startswith(":"):
-            network_split_info_extra = network_info_split[1].strip().split(":")
-
-            longname = network_split_info_extra[0].strip()
-            network_dict["longname"] = longname
-
-            implementation = (
-                network_split_info_extra[1].strip().replace("implemented in", "")
-            )
-            if implementation:
-                network_dict["implemented_in"] = implementation.strip().split()
-
-        networks_dict[network_number] = copy.deepcopy(network_dict)
-    version_info_dict["networks"] = networks_dict if networks_dict else None
-
-    ##########################
-    # Isotopes:
-    # Split off
-    isotopes = {el for el in cleaned if el.startswith("Isotope ")}
-    cleaned = cleaned - isotopes
-
-    isotope_dict = {}
-    for el in isotopes:
-        split_info = el.split("Isotope ")[-1].strip().split(" is ")
-
-        isotope_info = split_info[-1]
-        name = isotope_info.split(" ")[0].strip()
-
-        # Get details
-        mass_g = float(
-            isotope_info.split(",")[0].split("(")[1].split("=")[-1][:-2].strip()
-        )
-        mass_amu = float(
-            isotope_info.split(",")[0].split("(")[-1].split("=")[-1].strip()
-        )
-        mass_mev = float(
-            isotope_info.split(",")[-3].split("=")[-1].replace(")", "").strip()
-        )
-        A = int(isotope_info.split(",")[-1].strip().split("=")[-1].replace(")", ""))
-        Z = int(isotope_info.split(",")[-2].strip().split("=")[-1])
-
-        #
-        isotope_dict[int(split_info[0])] = {
-            "name": name,
-            "Z": Z,
-            "A": A,
-            "mass_mev": mass_mev,
-            "mass_g": mass_g,
-            "mass_amu": mass_amu,
-        }
-    version_info_dict["isotopes"] = isotope_dict if isotope_dict else None
-
-    ##########################
-    # Arg pairs:
-    # Split off
-    argpairs = set([el for el in cleaned if el.startswith("ArgPair")])
-    cleaned = cleaned - argpairs
-
-    argpair_dict = {}
-    for el in sorted(argpairs):
-        split_info = el.split("ArgPair ")[-1].split(" ")
-
-        if not argpair_dict.get(split_info[0], None):
-            argpair_dict[split_info[0]] = {split_info[1]: split_info[2]}
-        else:
-            argpair_dict[split_info[0]][split_info[1]] = split_info[2]
-
-    version_info_dict["argpairs"] = argpair_dict if argpair_dict else None
-
-    ##########################
-    # ensembles:
-    # Split off
-    ensembles = {el for el in cleaned if el.startswith("Ensemble")}
-    cleaned = cleaned - ensembles
-
-    ensemble_dict = {}
-    ensemble_filter_dict = {}
-    for el in ensembles:
-        split_info = el.split("Ensemble ")[-1].split(" is ")
-
-        if len(split_info) > 1:
-            if not split_info[0].startswith("filter"):
-                ensemble_dict[int(split_info[0])] = split_info[-1]
-            else:
-                filter_no = int(split_info[0].replace("filter ", ""))
-                ensemble_filter_dict[filter_no] = split_info[-1]
-
-    version_info_dict["ensembles"] = ensemble_dict if ensemble_dict else None
-    version_info_dict["ensemble_filters"] = (
-        ensemble_filter_dict if ensemble_filter_dict else None
-    )
-
-    ##########################
-    # macros:
-    # Split off
-    macros = {el for el in cleaned if el.startswith("macroxyz")}
-    cleaned = cleaned - macros
-
-    param_type_dict = {
-        "STRING": str,
-        "FLOAT": float,
-        "MACRO": str,
-        "INT": int,
-        "LONG_INT": int,
-        "UINT": int,
-    }
-
-    macros_dict = {}
-    for el in macros:
-        split_info = el.split("macroxyz ")[-1].split(" : ")
-        param_type = split_info[0]
-
-        new_split = "".join(split_info[1:]).split(" is ")
-        param_name = new_split[0]
-        param_value = " is ".join(new_split[1:])
-
-        # Sometimes the macros have extra information behind it. Needs an update in outputting by binary_c
-        try:
-            macros_dict[param_name] = param_type_dict[param_type](param_value)
-        except ValueError:
-            macros_dict[param_name] = str(param_value)
-    version_info_dict["macros"] = macros_dict if macros_dict else None
-
-    ##########################
-    # Elements:
-    # Split off:
-    elements = {el for el in cleaned if el.startswith("Element")}
-    cleaned = cleaned - elements
-
-    # Fill dict:
-    elements_dict = {}
-    for el in elements:
-        split_info = el.split("Element ")[-1].split(" : ")
-        name_info = split_info[0].split(" is ")
-
-        # get isotope info
-        isotopes = {}
-        if not split_info[-1][0] == "0":
-            isotope_string = split_info[-1].split(" = ")[-1]
-            isotopes = {
-                int(split_isotope.split("=")[0]): split_isotope.split("=")[1]
-                for split_isotope in isotope_string.split(" ")
-            }
-
-        elements_dict[int(name_info[0])] = {
-            "name": name_info[-1],
-            "atomic_number": int(name_info[0]),
-            "amt_isotopes": len(isotopes),
-            "isotopes": isotopes,
-        }
-    version_info_dict["elements"] = elements_dict if elements_dict else None
-
-    ##########################
-    # dt_limits:
-    # split off
-    dt_limits = {el for el in cleaned if el.startswith("DTlimit")}
-    cleaned = cleaned - dt_limits
-
-    # Fill dict
-    dt_limits_dict = {}
-    for el in dt_limits:
-        split_info = el.split("DTlimit ")[-1].split(" : ")
-        dt_limits_dict[split_info[1].strip()] = {
-            "index": int(split_info[0]),
-            "value": float(split_info[-1]),
-        }
-
-    version_info_dict["dt_limits"] = dt_limits_dict if dt_limits_dict else None
-
-    ##########################
-    # Nucleosynthesis sources:
-    # Split off
-    nucsyn_sources = {el for el in cleaned if el.startswith("Nucleosynthesis")}
-    cleaned = cleaned - nucsyn_sources
-
-    # Fill dict
-    nucsyn_sources_dict = {}
-    for el in nucsyn_sources:
-        split_info = el.split("Nucleosynthesis source")[-1].strip().split(" is ")
-        nucsyn_sources_dict[int(split_info[0])] = split_info[-1]
-
-    version_info_dict["nucleosynthesis_sources"] = (
-        nucsyn_sources_dict if nucsyn_sources_dict else None
-    )
-
-    ##########################
-    # miscellaneous:
-    # All those that I didn't catch with the above filters. Could try to get some more out though.
-    # TODO: filter a bit more.
-
-    misc_dict = {}
-
-    # Filter out git revision
-    git_revision = [el for el in cleaned if el.startswith("git revision")]
-    misc_dict["git_revision"] = (
-        git_revision[0].split("git revision ")[-1].replace('"', "")
-    )
-    cleaned = cleaned - set(git_revision)
-
-    # filter out git url
-    git_url = [el for el in cleaned if el.startswith("git URL")]
-    misc_dict["git_url"] = git_url[0].split("git URL ")[-1].replace('"', "")
-    cleaned = cleaned - set(git_url)
-
-    # filter out version
-    version = [el for el in cleaned if el.startswith("Version")]
-    misc_dict["version"] = str(version[0].split("Version ")[-1])
-    cleaned = cleaned - set(version)
-
-    git_branch = [el for el in cleaned if el.startswith("git branch")]
-    misc_dict["git_branch"] = git_branch[0].split("git branch ")[-1].replace('"', "")
-    cleaned = cleaned - set(git_branch)
-
-    build = [el for el in cleaned if el.startswith("Build")]
-    misc_dict["build"] = build[0].split("Build: ")[-1].replace('"', "")
-    cleaned = cleaned - set(build)
-
-    email = [el for el in cleaned if el.startswith("Email")]
-    misc_dict["email"] = email[0].split("Email ")[-1].split(",")
-    cleaned = cleaned - set(email)
-
-    other_items = set([el for el in cleaned if " is " in el])
-    cleaned = cleaned - other_items
-
-    for el in other_items:
-        split = el.split(" is ")
-        key = split[0].strip()
-        val = " is ".join(split[1:]).strip()
-        misc_dict[key] = val
-
-    misc_dict["uncaught"] = list(cleaned)
-
-    version_info_dict["miscellaneous"] = misc_dict if misc_dict else None
-    return version_info_dict
-
-
 ########################################################
 # binary_c output functions
 ########################################################
@@ -1078,7 +786,7 @@ def get_help(
 
     Will check whether it is a valid parameter.
 
-    Binary_c will output things in the following order;
+    Binary_c will output things in the following order:
     - Did you mean?
     - binary_c help for variable
     - default
@@ -1086,9 +794,6 @@ def get_help(
 
     This function reads out that structure and catches the different components of this output
 
-    Tasks:
-        - TODO: consider not returning None, but return empty dict
-
     Args:
         param_name: name of the parameter that you want info from. Will get checked whether its a
             valid parameter name
@@ -1097,8 +802,8 @@ def get_help(
         parameter isn't valid
 
     Returns:
-        Dictionary containing the help info. This dictionary contains 'parameter_name',
-        'parameter_value_input_type', 'description', optionally 'macros'
+        Dictionary containing the help info. This dictionary contains `parameter_name`,
+        `parameter_value_input_type`, `description`, optionally `macros`
     """
 
     available_arg_keys = get_arg_keys()
@@ -1166,20 +871,20 @@ def get_help(
             help_info_dict["macros"] = macros
 
         if print_help:
-            for key in help_info_dict:
-                print("{}:\n\t{}".format(key, help_info_dict[key]))
+            for key, value in help_info_dict.items():
+                print("{}:\n\t{}".format(key, value))
 
         return help_info_dict
 
-    else:
-        if not fail_silently:
-            print(
-                "{} is not a valid parameter name. Please choose from the \
-                following parameters:\n\t{}".format(
-                    param_name, list(available_arg_keys)
-                )
+    if not fail_silently:
+        print(
+            "{} is not a valid parameter name. Please choose from the \
+            following parameters:\n\t{}".format(
+                param_name, list(available_arg_keys)
             )
-        return None
+        )
+
+    return {}
 
 
 def get_help_all(print_help: bool = True) -> dict:
@@ -1208,11 +913,11 @@ def get_help_all(print_help: bool = True) -> dict:
     help_all_dict = {}
 
     # Select the section name and the contents of that section. Note, not all sections have content!
-    for i in range(len(section_nums)):
+    for i, section_num in enumerate(section_nums):
         if not i == len(section_nums) - 1:
-            params = cleaned[section_nums[i] + 1 : section_nums[i + 1]]
+            params = cleaned[section_num + 1 : section_nums[i + 1]]
         else:
-            params = cleaned[section_nums[i] + 1 : len(cleaned)]
+            params = cleaned[section_num + 1 : len(cleaned)]
         section_name = (
             cleaned[section_nums[i]]
             .lstrip("#####")
@@ -1310,21 +1015,14 @@ def get_help_super(print_help: bool = False, fail_silently: bool = True) -> dict
 
     # Get help_all information
     help_all_dict = get_help_all(print_help=False)
-    for section_name in help_all_dict:
-        section = help_all_dict[section_name]
-
-        # print(section_name)
-        # for parameter_name in section["parameters"].keys():
-        #     print("\t", parameter_name)
 
+    #
     help_all_super_dict = help_all_dict.copy()
 
     # Loop over all sections and stuff
-    for section_name in help_all_dict:
+    for section_name, section in help_all_dict.items():
         # Skipping the section i/o because that one shouldn't be available to python anyway
         if not section_name == "i/o":
-            section = help_all_dict[section_name]
-
             for parameter_name in section["parameters"].keys():
                 parameter = section["parameters"][parameter_name]
 
@@ -1339,7 +1037,7 @@ def get_help_super(print_help: bool = False, fail_silently: bool = True) -> dict
                     # check whether the descriptions of help_all and detailed help are the same
                     if not fail_silently:
                         if not parameter["description"] == detailed_help["description"]:
-                            print(json.dumps(parameter, indent=4))
+                            print(json.dumps(parameter, indent=4, ensure_ascii=False))
 
                     ## put values into help all super dict
                     # input type
@@ -1351,13 +1049,13 @@ def get_help_super(print_help: bool = False, fail_silently: bool = True) -> dict
                     parameter["default"] = detailed_help["default"]
 
                     # macros
-                    if "macros" in detailed_help.keys():
+                    if "macros" in detailed_help:
                         parameter["macros"] = detailed_help["macros"]
 
                 section["parameters"][parameter_name] = parameter
 
     if print_help:
-        print(json.dumps(help_all_super_dict, indent=4))
+        print(json.dumps(help_all_super_dict, indent=4, ensure_ascii=False))
 
     return help_all_super_dict
 
@@ -1369,8 +1067,11 @@ def make_build_text() -> str:
     Returns:
         string containing information about the build and the git branch
     """
+    from binarycpython.utils.grid import Population
 
-    version_info = return_binary_c_version_info(parsed=True)
+    version_pop = Population()
+    version_info = version_pop.return_binary_c_version_info(parsed=True)
+    # version_info = return_binary_c_version_info(parsed=True)
 
     git_revision = version_info["miscellaneous"]["git_revision"]
     git_branch = version_info["miscellaneous"]["git_branch"]
@@ -1390,11 +1091,8 @@ def write_binary_c_parameter_descriptions_to_rst_file(output_file: str) -> None:
     """
     Function that calls the get_help_super() to get the help text/descriptions for all the
     parameters available in that build.
-    Writes the results to a .rst file that can be included in the docs.
 
-    Tasks:
-        - TODO: add the specific version git branch, git build, git commit, and binary_c version to
-            this document
+    Writes the results to a .rst file that can be included in the docs.
 
     Args:
         output_file: name of the output .rst file containing the ReStructuredText formatted output
@@ -1407,10 +1105,13 @@ def write_binary_c_parameter_descriptions_to_rst_file(output_file: str) -> None:
     build_info = make_build_text()
 
     if not output_file.endswith(".rst"):
-        print("Filename doesn't end with .rst, please provide a proper filename")
-        return None
+        raise ValueError(
+            "Filename ({}) doesn't end with .rst, please provide a proper filename.".format(
+                output_file
+            )
+        )
 
-    with open(output_file, "w") as f:
+    with open(output_file, "w", encoding="utf-8") as f:
 
         print("Binary\\_c parameters", file=f)
         print("{}".format("=" * len("Binary\\_c parameters")), file=f)
@@ -1425,13 +1126,19 @@ def write_binary_c_parameter_descriptions_to_rst_file(output_file: str) -> None:
         for el in arguments_dict.keys():
             print("Section: {}".format(el), file=f)
             print("{}\n".format("-" * len("Section: {}".format(el))), file=f)
-            # print(arguments_dict[el]['parameters'].keys())
 
             for arg in arguments_dict[el]["parameters"].keys():
                 argdict = arguments_dict[el]["parameters"][arg]
 
                 print("| **Parameter**: {}".format(argdict["param_name"]), file=f)
-                print("| **Description**: {}".format(argdict["description"]), file=f)
+                print(
+                    "| **Description**: {}".format(
+                        argdict["description"].replace(
+                            "|Rout/Rin-1|", "abs(Rout/Rin-1)"
+                        )
+                    ),
+                    file=f,
+                )
                 if "parameter_value_input_type" in argdict:
                     print(
                         "| **Parameter input type**: {}".format(
@@ -1453,14 +1160,14 @@ def write_binary_c_parameter_descriptions_to_rst_file(output_file: str) -> None:
 ########################################################
 
 
-def load_logfile(logfile: str) -> None:
+def load_logfile(logfile: str) -> None: # pragma: no cover
     """
     Experimental function that parses the generated log file of binary_c.
 
     This function is not finished and shouldn't be used yet.
 
     Tasks:
-        - TODO:
+        - TODO: fix this function
 
     Args:
         - logfile: filename of the log file you want to parse
@@ -1469,7 +1176,7 @@ def load_logfile(logfile: str) -> None:
 
     """
 
-    with open(logfile, "r") as file:
+    with open(logfile, "r", encoding="utf-8") as file:
         logfile_data = file.readlines()
 
     time_list = []
@@ -1502,3 +1209,15 @@ def load_logfile(logfile: str) -> None:
         event_list.append(" ".join(split_line[9:]))
 
     print(event_list)
+
+def quotewrap(list):
+    """
+    Given a list, wrap each item in double quotes and return the new list
+    """
+    return ['"' + _x + '"' for _x in list]
+
+def command_string_from_list(list):
+    """
+    Given a list, turn it into a quoted command string
+    """
+    return ' '.join(quotewrap(list))
