@@ -1,7 +1,7 @@
 """
-Module that contains functionality to plot some properties of (binary) systems. 
+Module that contains functionality to plot some properties of (binary) systems.
 
-Different routines are defined here to plot orbits, masses, angular momenta etc. 
+Different routines are defined here to plot orbits, masses, angular momenta etc.
 
 Structure of file:
     custom logging strings
@@ -13,8 +13,8 @@ All the loose components here can of course be used in other routines if you wan
 
 There is no pre loaded matplotlib rc, you should do that yourself
 
-These plotting routines are designed for binary systems, and admittedly they are here mostly for 
-inspirational purposes, since one would probably want to customise the plots. 
+These plotting routines are designed for binary systems, and admittedly they are here mostly for
+inspirational purposes, since one would probably want to customise the plots.
 Regardless, having some plotting routines in here seemed like a nice idea
 
 Tasks
@@ -164,10 +164,10 @@ def plot_HR_diagram(
     prefactor = (1 / (4 * math.pi * omega_sb)) ** (1.0 / 4)
 
     if show_stellar_types:
-        fig, ax = plt.subplots(ncols=1, nrows=1, figsize=(20, 20))
-        fig, colors = add_stellar_types_bar(df, fig, ax_index=-1, only_colorbar=True)
+        fig, _ = plt.subplots(ncols=1, nrows=1, figsize=(20, 20))
+        # fig, colors = add_stellar_types_bar(df, fig, ax_index=-1, only_colorbar=True)
     else:
-        fig, ax = plt.subplots(ncols=1, nrows=1, figsize=(20, 20))
+        fig, _ = plt.subplots(ncols=1, nrows=1, figsize=(20, 20))
 
     df = df.assign(
         teff_1=prefactor
@@ -229,8 +229,7 @@ def plot_HR_diagram(
     # Show or return
     if show_plot:
         plt.show()
-    else:
-        return fig
+    return fig
 
 
 def plot_orbit(df, show_stellar_types: bool = False, show_plot: bool = True):
@@ -254,10 +253,10 @@ def plot_orbit(df, show_stellar_types: bool = False, show_plot: bool = True):
     """
 
     if show_stellar_types:
-        fig, ax = plt.subplots(ncols=1, nrows=4, figsize=(20, 10))
+        fig, _ = plt.subplots(ncols=1, nrows=4, figsize=(20, 10))
         fig.subplots_adjust(hspace=0)
     else:
-        fig, ax = plt.subplots(ncols=1, nrows=3, figsize=(20, 10), sharex=True)
+        fig, _ = plt.subplots(ncols=1, nrows=3, figsize=(20, 10), sharex=True)
         fig.subplots_adjust(hspace=0)
 
     #
@@ -286,8 +285,7 @@ def plot_orbit(df, show_stellar_types: bool = False, show_plot: bool = True):
     # Show or return
     if show_plot:
         plt.show()
-    else:
-        return fig
+    return fig
 
 
 def plot_masses(df, show_stellar_types: bool = False, show_plot: bool = True):
@@ -322,10 +320,10 @@ def plot_masses(df, show_stellar_types: bool = False, show_plot: bool = True):
     """
 
     if show_stellar_types:
-        fig, ax = plt.subplots(ncols=1, nrows=2, figsize=(20, 10))
+        fig, _ = plt.subplots(ncols=1, nrows=2, figsize=(20, 10))
         fig.subplots_adjust(hspace=0)
     else:
-        fig, ax = plt.subplots(ncols=1, nrows=1, figsize=(20, 10))
+        fig, _ = plt.subplots(ncols=1, nrows=1, figsize=(20, 10))
 
     max_total_mass = (
         df["pms_mass_1"].values.tolist()[0] + df["pms_mass_2"].values.tolist()[0]
@@ -393,14 +391,16 @@ def plot_masses(df, show_stellar_types: bool = False, show_plot: bool = True):
     # Show or return
     if show_plot:
         plt.show()
-    else:
-        return fig
+    return fig
 
 
 # Define the parse functions for the plotting routines
 def dummy():
-    """Placeholder"""
-    pass
+    """
+    Placeholder function
+    """
+
+    return None
 
 
 def parse_function_hr_diagram(output: str):
@@ -543,11 +543,6 @@ def plot_system(plot_type, **kwargs):
         - orbit_evolution
         - hr_diagram
 
-    Tasks:
-        - TODO: Complex Function!
-        - TODO: make sure this way of passing args works correctly.
-        - TODO: make the plotting specific keywords available via the inspect stuff
-
     All keywords are considered kwargs, except for plot_type
 
     Args:
@@ -602,15 +597,15 @@ def plot_system(plot_type, **kwargs):
     #   run_system will handle the rest
     run_system_arg_dict = {}
 
-    for key in kwargs.keys():
+    for key, value in kwargs.items():
         if key == "show_plot":
-            show_plot = kwargs[key]
+            show_plot = value
         elif key == "show_stellar_types":
-            show_stellar_types = kwargs[key]
+            show_stellar_types = value
 
         # The rest will be passed to run_system
         else:
-            run_system_arg_dict[key] = kwargs[key]
+            run_system_arg_dict[key] = value
 
     # TODO: When a list of plot_types is passed, make it so that the strings are chained,
     #   and that the output of the binary_c call is handled by multiple parsers
@@ -628,22 +623,4 @@ def plot_system(plot_type, **kwargs):
         binary_c_output_df, show_plot=show_plot, show_stellar_types=show_stellar_types
     )
 
-    if not show_plot:
-        return fig
-
-
-# from david_phd_functions.plotting.custom_mpl_settings import load_mpl_rc
-# load_mpl_rc()
-
-# fig = plot_system(
-#     plot_type="mass_evolution",
-#     M_1=10,
-#     M_2=5,
-#     separation=1000000,
-#     orbital_period=100000000,
-#     max_evolution_time=15000,
-#     show_plot=True,
-# )
-
-# fig.axes[0].set_xlim(0, 150)
-# plt.show()
+    return fig
